@@ -1,6 +1,7 @@
 from ._getter import FEATURES
 from ._getter import get_feature
 from .features import Barrier
+from .features import Empty
 from .features import ExpiryTime
 from .features import LogMoneyness
 from .features import MaxLogMoneyness
@@ -9,4 +10,4 @@ from .features import ModuleOutput
 from .features import Moneyness
 from .features import PrevHedge
 from .features import Volatility
-from .features import Zero
+from .features import Zeros
