@@ -1,9 +1,12 @@
 from abc import ABC
 from abc import abstractmethod
+from typing import List
 from typing import Optional
 from typing import TypeVar
+from typing import no_type_check
 
 import torch
+from torch import Tensor
 
 T = TypeVar("T", bound="Instrument")
 
@@ -12,6 +15,7 @@ class Instrument(ABC):
     """Base class for all financial instruments."""
 
     @abstractmethod
+    @no_type_check
     def simulate(self, n_paths: int, time_horizon: float, **kwargs) -> None:
         """Simulate time series associated with the instrument itself
         (for a primary instrument) or its underlier (for a derivative)
@@ -30,8 +34,8 @@ class Instrument(ABC):
         """Performs dtype and/or device conversion of the buffers associated to
         the instument.
 
-        A `torch.dtype` and `torch.device` are inferred from the arguments of
-        `self.to(*args, **kwargs)`.
+        A ``torch.dtype`` and ``torch.device`` are inferred from the arguments of
+        ``self.to(*args, **kwargs)``.
 
         Args:
             dtype (torch.dtype): Desired floating point type of the floating point
@@ -61,48 +65,46 @@ class Instrument(ABC):
             device (torch.device): The destination GPU device.
                 Defaults to the current CUDA device.
         """
-        return self.to(
-            torch.device("cuda:{}".format(device) if device is not None else "cuda")
-        )
+        return self.to(torch.device(f"cuda:{device}" if device is not None else "cuda"))
 
     def double(self: T) -> T:
-        """`self.double()` is equivalent to `self.to(torch.float64)`.
+        """It is equivalent to ``self.to(torch.float64)``.
         See :func:`to()`.
         """
         return self.to(torch.float64)
 
     def float(self: T) -> T:
-        """`self.float()` is equivalent to `self.to(torch.float32)`.
+        """It is equivalent to ``self.to(torch.float32)``.
         See :func:`to()`.
         """
         return self.to(torch.float32)
 
     def half(self: T) -> T:
-        """`self.half()` is equivalent to `self.to(torch.float16)`.
+        """It is equivalent to ``self.to(torch.float16)``.
         See :func:`to()`.
         """
         return self.to(torch.float16)
 
     def bfloat16(self: T) -> T:
-        """`self.bfloat16()` is equivalent to `self.to(torch.bfloat16)`.
+        """It is equivalent to ``self.to(torch.bfloat16)``.
         See :func:`to()`.
         """
         return self.to(torch.bfloat16)
 
     @property
-    def dinfo(self) -> list:
-        """Returns list of strings that tell `dtype` and `device` of `self`.
+    def dinfo(self) -> List[str]:
+        """Returns list of strings that tell ``dtype`` and ``device`` of self.
 
-        Intended to be used in `__repr__`.
+        Intended to be used in :func:`__repr__`.
 
-        If `dtype` (`device`) is the one specified in default type,
-        `dinfo` will not have the information of it.
+        If ``dtype`` (``device``) is the one specified in default type,
+        ``dinfo`` will not have the information of it.
 
         Returns:
             list[str]
         """
-        # Implementation here refers to the function `_str_intern` in
-        # `pytorch/_tensor_str.py`.
+        # Implementation here refers to the function _str_intern in
+        # pytorch/_tensor_str.py.
 
         dinfo = []
 
