@@ -1,7 +1,13 @@
+from typing import Optional
+
 import torch
 from torch import Tensor
 
+from pfhedge._utils.doc import set_attr_and_docstring
+from pfhedge._utils.doc import set_docstring
+
 from ...nn.functional import european_payoff
+from ..primary.base import Primary
 from .base import Derivative
 from .base import OptionMixin
 
@@ -35,11 +41,11 @@ class EuropeanOption(Derivative, OptionMixin):
         maturity (float, default=20/250): The maturity of the option.
         dtype (torch.dtype, optional): Desired device of returned tensor.
             Default: If None, uses a global default
-            (see `torch.set_default_tensor_type()`).
+            (see ``torch.set_default_tensor_type()``).
         device (torch.device, optional): Desired device of returned tensor.
             Default: if None, uses the current device for the default tensor type
-            (see `torch.set_default_tensor_type()`).
-            `device` will be the CPU for CPU tensor types and
+            (see ``torch.set_default_tensor_type()``).
+            ``device`` will be the CPU for CPU tensor types and
             the current CUDA device for CUDA tensor types.
 
     Attributes:
@@ -57,27 +63,45 @@ class EuropeanOption(Derivative, OptionMixin):
         >>> derivative = EuropeanOption(BrownianStock(), maturity=5/250)
         >>> derivative.simulate(n_paths=2)
         >>> derivative.underlier.spot
-        tensor([[1.0000, 1.0016, 1.0044, 1.0073, 0.9930],
-                [1.0000, 1.0282, 1.0199, 1.0258, 1.0292]])
+        tensor([[1.0000, 1.0016, 1.0044, 1.0073, 0.9930, 0.9906],
+                [1.0000, 0.9919, 0.9976, 1.0009, 1.0076, 1.0179]])
         >>> derivative.payoff()
-        tensor([0.0000, 0.0292])
+        tensor([0.0000, 0.0179])
 
-        Using custom `dtype` and `device`.
+        Using custom ``dtype`` and ``device``.
 
         >>> derivative = EuropeanOption(BrownianStock())
         >>> derivative.to(dtype=torch.float64, device="cuda:0")
         EuropeanOption(..., dtype=torch.float64, device='cuda:0')
+
+        Make ``self`` a listed derivative.
+
+        >>> from pfhedge.nn import BlackScholes
+        >>>
+        >>> pricer = lambda derivative: BlackScholes(derivative).price(
+        ...     log_moneyness=derivative.log_moneyness(),
+        ...     expiry_time=derivative.time_to_maturity(),
+        ...     volatility=derivative.ul().volatility)
+        >>> derivative = EuropeanOption(BrownianStock(), maturity=5/250)
+        >>> derivative.list(pricer, cost=1e-4)
+        >>> derivative.simulate(n_paths=2)
+        >>> derivative.ul().spot
+        tensor([[1.0000, 0.9788, 0.9665, 0.9782, 0.9947, 1.0049],
+                [1.0000, 0.9905, 1.0075, 1.0162, 1.0119, 1.0220]])
+        >>> derivative.spot
+        tensor([[0.0113, 0.0028, 0.0006, 0.0009, 0.0028, 0.0049],
+                [0.0113, 0.0060, 0.0130, 0.0180, 0.0131, 0.0220]])
     """
 
     def __init__(
         self,
-        underlier,
+        underlier: Primary,
         call: bool = True,
         strike: float = 1.0,
         maturity: float = 20 / 250,
-        dtype: torch.dtype = None,
-        device: torch.device = None,
-    ):
+        dtype: Optional[torch.dtype] = None,
+        device: Optional[torch.device] = None,
+    ) -> None:
         super().__init__()
         self.underlier = underlier
         self.call = call
@@ -87,7 +111,7 @@ class EuropeanOption(Derivative, OptionMixin):
         self.to(dtype=dtype, device=device)
 
     def __repr__(self):
-        params = [f"{self.underlier.__class__.__name__}(...)"]
+        params = [f"{self.ul().__class__.__name__}(...)"]
         if not self.call:
             params.append(f"call={self.call}")
         params.append(f"strike={self.strike}")
@@ -100,14 +124,10 @@ class EuropeanOption(Derivative, OptionMixin):
 
 
 # Assign docstrings so they appear in Sphinx documentation
-EuropeanOption.simulate = Derivative.simulate
-EuropeanOption.simulate.__doc__ = Derivative.simulate.__doc__
-EuropeanOption.to = Derivative.to
-EuropeanOption.to.__doc__ = Derivative.to.__doc__
-EuropeanOption.payoff.__doc__ = Derivative.payoff.__doc__
-EuropeanOption.moneyness = OptionMixin.moneyness
-EuropeanOption.moneyness.__doc__ = OptionMixin.moneyness.__doc__
-EuropeanOption.log_moneyness = OptionMixin.log_moneyness
-EuropeanOption.log_moneyness.__doc__ = OptionMixin.log_moneyness.__doc__
-EuropeanOption.time_to_maturity = OptionMixin.time_to_maturity
-EuropeanOption.time_to_maturity.__doc__ = OptionMixin.time_to_maturity.__doc__
+set_attr_and_docstring(EuropeanOption, "simulate", Derivative.simulate)
+set_attr_and_docstring(EuropeanOption, "to", Derivative.to)
+set_attr_and_docstring(EuropeanOption, "ul", Derivative.ul)
+set_docstring(EuropeanOption, "payoff", Derivative.payoff)
+set_attr_and_docstring(EuropeanOption, "moneyness", OptionMixin.moneyness)
+set_attr_and_docstring(EuropeanOption, "log_moneyness", OptionMixin.log_moneyness)
+set_attr_and_docstring(EuropeanOption, "time_to_maturity", OptionMixin.time_to_maturity)
