@@ -1,4 +1,6 @@
 from math import ceil
+from typing import Optional
+from typing import Union
 
 import torch
 import torch.nn.functional as fn
@@ -14,8 +16,8 @@ def european_payoff(input: Tensor, call: bool = True, strike: float = 1.0) -> Te
         strike (float, default=1.0): The strike price of the option.
 
     Shape:
-        - input: :math:`(*, T)`, where, :math:`T` stands for the number of time steps and
-          :math:`*` means any number of additional dimensions.
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
         - output: :math:`(*)`
 
     Returns:
@@ -36,8 +38,8 @@ def lookback_payoff(input: Tensor, call: bool = True, strike: float = 1.0) -> Te
         strike (float, default=1.0): The strike price of the option.
 
     Shape:
-        - input: :math:`(*, T)`, where, :math:`T` stands for the number of time steps and
-          :math:`*` means any number of additional dimensions.
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
         - output: :math:`(*)`
 
     Returns:
@@ -60,8 +62,8 @@ def american_binary_payoff(
         strike (float, default=1.0): The strike price of the option.
 
     Shape:
-        - input: :math:`(*, T)`, where, :math:`T` stands for the number of time steps and
-          :math:`*` means any number of additional dimensions.
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
         - output: :math:`(*)`
 
     Returns:
@@ -84,8 +86,8 @@ def european_binary_payoff(
         strike (float, default=1.0): The strike price of the option.
 
     Shape:
-        - input: :math:`(*, T)`, where, :math:`T` stands for the number of time steps and
-          :math:`*` means any number of additional dimensions.
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
         - output: :math:`(*)`
 
     Returns:
@@ -104,7 +106,7 @@ def exp_utility(input: Tensor, a: float = 1.0) -> Tensor:
 
     .. math ::
 
-        u(x) = -\\exp(-a x) \,.
+        u(x) = -\\exp(-a x) \\,.
 
     Args:
         input (torch.Tensor): The input tensor.
@@ -114,10 +116,10 @@ def exp_utility(input: Tensor, a: float = 1.0) -> Tensor:
     Returns:
         torch.Tensor
     """
-    return -torch.exp(-a * input)
+    return -(-a * input).exp()
 
 
-def isoelastic_utility(input: Tensor, a: float = 0.5) -> Tensor:
+def isoelastic_utility(input: Tensor, a: float) -> Tensor:
     """Applies an isoelastic utility function.
 
     An isoelastic utility function is defined as:
@@ -127,32 +129,32 @@ def isoelastic_utility(input: Tensor, a: float = 0.5) -> Tensor:
         u(x) = \\begin{cases}
         x^{1 - a} & a \\neq 1 \\\\
         \\log{x} & a = 1
-        \\end{cases} \,.
+        \\end{cases} \\,.
 
     Args:
         input (torch.Tensor): The input tensor.
-        a (float, default=0.5): Relative risk aversion coefficient of the isoelastic
+        a (float): Relative risk aversion coefficient of the isoelastic
             utility.
 
     Returns:
         torch.Tensor
     """
     if a == 1.0:
-        return torch.log(input)
+        return input.log()
     else:
-        return torch.pow(input, exponent=1.0 - a)
+        return input ** (1.0 - a)
 
 
-def topp(input, p: float, dim: int = None, largest: bool = True):
-    """Returns the largest `p * N` elements of the given input tensor,
-    where `N` stands for the total number of elements in the input tensor.
+def topp(input: Tensor, p: float, dim: Optional[int] = None, largest: bool = True):
+    """Returns the largest :math:`p * N` elements of the given input tensor,
+    where :math:`N` stands for the total number of elements in the input tensor.
 
-    If `dim` is not given, the last dimension of the `input` is chosen.
+    If ``dim`` is not given, the last dimension of the ``input`` is chosen.
 
-    If `largest` is `False` then the smallest elements are returned.
+    If ``largest`` is ``False`` then the smallest elements are returned.
 
-    A namedtuple of `(values, indices)` is returned, where the `indices` are the indices
-    of the elements in the original `input` tensor.
+    A namedtuple of ``(values, indices)`` is returned, where the ``indices``
+    are the indices of the elements in the original ``input`` tensor.
 
     Args:
         input (torch.Tensor): The input tensor.
@@ -166,6 +168,8 @@ def topp(input, p: float, dim: int = None, largest: bool = True):
 
     Examples:
 
+        >>> from pfhedge.nn.functional import topp
+        >>>
         >>> input = torch.arange(1.0, 6.0)
         >>> input
         tensor([1., 2., 3., 4., 5.])
@@ -175,23 +179,23 @@ def topp(input, p: float, dim: int = None, largest: bool = True):
         indices=tensor([4, 3, 2]))
     """
     if dim is None:
-        return torch.topk(input, ceil(p * input.numel()), largest=largest)
+        return input.topk(ceil(p * input.numel()), largest=largest)
     else:
-        return torch.topk(input, ceil(p * input.size()[dim]), dim=dim, largest=largest)
+        return input.topk(ceil(p * input.size()[dim]), dim=dim, largest=largest)
 
 
-def expected_shortfall(input: Tensor, p: float, dim=None) -> Tensor:
+def expected_shortfall(input: Tensor, p: float, dim: Optional[int] = None) -> Tensor:
     """Returns the expected shortfall of the given input tensor.
 
     Args:
         input (torch.Tensor): The input tensor.
         p (float): Quantile level.
         dim (int, optional): The dimension to sort along.
-        largest (bool, default=True): Controls whether to return largest or smallest
-            elements.
 
     Examples:
 
+        >>> from pfhedge.nn.functional import expected_shortfall
+        >>>
         >>> input = -torch.arange(1., 6.)
         >>> expected_shortfall(input, 3 / 5)
         tensor(4.)
@@ -206,9 +210,12 @@ def expected_shortfall(input: Tensor, p: float, dim=None) -> Tensor:
 
 
 def leaky_clamp(
-    input: Tensor, min: Tensor = None, max: Tensor = None, clamped_slope: float = 0.01
+    input: Tensor,
+    min: Optional[Tensor] = None,
+    max: Optional[Tensor] = None,
+    clamped_slope: float = 0.01,
 ) -> Tensor:
-    """Leakily clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+    """Leakily clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
     The bounds :math:`\\min` and :math:`\\max` can be tensors.
 
@@ -218,23 +225,75 @@ def leaky_clamp(
 
     if min is not None:
         min = torch.as_tensor(min)
-        x = torch.max(x, min + clamped_slope * (x - min))
+        x = x.maximum(min + clamped_slope * (x - min))
 
     if max is not None:
         max = torch.as_tensor(max)
-        x = torch.min(x, max + clamped_slope * (x - max))
+        x = x.minimum(max + clamped_slope * (x - max))
 
     if min is not None and max is not None:
-        x = torch.where(min <= max, x, (min + max) / 2)
+        x = x.where(min <= max, (min + max) / 2)
 
     return x
 
 
-def clamp(input: Tensor, min: Tensor = None, max: Tensor = None) -> Tensor:
-    """Clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+def clamp(
+    input: Tensor, min: Optional[Tensor] = None, max: Optional[Tensor] = None
+) -> Tensor:
+    """Clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
     The bounds :math:`\\min` and :math:`\\max` can be tensors.
 
     See :class:`pfhedge.nn.Clamp` for details.
     """
     return leaky_clamp(input, min=min, max=max, clamped_slope=0.0)
+
+
+def realized_variance(input: Tensor, dt: Union[Tensor, float]) -> Tensor:
+    """Returns the realized variance of the price.
+
+    Realized variance :math:`\\sigma^2` of the stock price :math:`S` is defined as:
+
+    .. math ::
+
+        \\sigma^2 = \\frac{1}{T - 1} \\sum_{i = 1}^{T - 1}
+        \\frac{1}{dt} \\log(S_{i + 1} / S_i)^2
+
+    where :math:`T` is the number of time steps.
+
+    .. note ::
+
+        The mean of log return is assumed to be zero.
+
+    Args:
+        input (torch.Tensor): The input tensor.
+        dt (torch.Tensor or float): The intervals of the time steps.
+
+    Shape:
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
+        - output: :math:`(*)`
+
+    Returns:
+        torch.Tensor
+    """
+    return input.log().diff(dim=-1).square().mean(dim=-1) / dt
+
+
+def realized_volatility(input: Tensor, dt: Union[Tensor, float]) -> Tensor:
+    """Returns the realized volatility of the price.
+    It is square root of :func:`realized_variance`.
+
+    Args:
+        input (torch.Tensor): The input tensor.
+        dt (torch.Tensor or float): The intervals of the time steps.
+
+    Shape:
+        - input: :math:`(*, T)` where :math:`T` stands for the number of time steps
+          and :math:`*` means any number of additional dimensions.
+        - output: :math:`(*)`
+
+    Returns:
+        torch.Tensor
+    """
+    return realized_variance(input, dt=dt).sqrt()
