@@ -1,5 +1,7 @@
 import abc
 from inspect import signature
+from typing import List
+from typing import no_type_check
 
 import torch
 from torch import Tensor
@@ -12,7 +14,7 @@ class BSModuleMixin(Module):
 
     Shape:
         - Input: :math:`(N, *, H_\\text{in})`, where :math:`*` means any number of
-          additional dimensions. See `inputs()` for the names of input features.
+          additional dimensions. See :func:`inputs` for the names of input features.
         - Output: :math:`(N, *, 1)`: All but the last dimension are the same shape
           as the input.
     """
@@ -22,7 +24,8 @@ class BSModuleMixin(Module):
 
         Args:
             input (torch.Tensor): The input tensor. Features are concatenated along
-                the last dimension. See `inputs()` for the names of the input features.
+                the last dimension.
+                See :func:`inputs()` for the names of the input features.
 
         Returns:
             torch.Tensor
@@ -30,6 +33,7 @@ class BSModuleMixin(Module):
         return self.delta(*(input[..., [i]] for i in range(input.size(-1))))
 
     @abc.abstractmethod
+    @no_type_check
     def delta(self, *args, **kwargs) -> Tensor:
         """Returns delta of the derivative.
 
@@ -37,7 +41,7 @@ class BSModuleMixin(Module):
             torch.Tensor
         """
 
-    def inputs(self) -> list:
+    def inputs(self) -> List[str]:
         """Returns the names of input features.
 
         Returns:
@@ -47,14 +51,8 @@ class BSModuleMixin(Module):
 
     @property
     def N(self) -> Normal:
-        """Returns normal distribution with zero mean and unit standard deviation.
-
-        It is almost the same with `torch.distibution.normal.Normal(0, 1)`, but has
-        a method `pdf` which stands for the partial distribution function.
-        """
-        normal = Normal(torch.tensor(0.0), torch.tensor(1.0))
-        setattr(normal, "pdf", lambda value: normal.log_prob(value).exp())
-        return normal
+        """Returns normal distribution with zero mean and unit standard deviation."""
+        return Normal(torch.tensor(0.0), torch.tensor(1.0))
 
     @staticmethod
     def d1(log_moneyness: Tensor, expiry_time: Tensor, volatility: Tensor) -> Tensor:
