@@ -3,6 +3,7 @@ from torch import Tensor
 
 import pfhedge.autogreek as autogreek
 from pfhedge._utils.bisect import bisect
+from pfhedge._utils.doc import set_attr_and_docstring
 
 from ._base import BSModuleMixin
 
@@ -17,7 +18,7 @@ class BSAmericanBinaryOption(BSModuleMixin):
 
     Shape:
         - Input: :math:`(N, *, 4)`, where :math:`*` means any number of additional
-          dimensions. See `inputs()` for the names of input features.
+          dimensions. See :func:`inputs` for the names of input features.
         - Output: :math:`(N, *, 1)`. Delta of the derivative.
           All but the last dimension are the same shape as the input.
 
@@ -28,7 +29,7 @@ class BSAmericanBinaryOption(BSModuleMixin):
 
     Examples:
 
-        The `forward` method returns delta of the derivative.
+        The ``forward`` method returns delta of the derivative.
 
         >>> from pfhedge.nn import BSAmericanBinaryOption
         >>>
@@ -43,6 +44,10 @@ class BSAmericanBinaryOption(BSModuleMixin):
         tensor([[1.1285],
                 [0.0000],
                 [0.0000]])
+
+    References:
+        Dai, M., 2000. A closed-form solution for perpetual American floating strike
+        lookback options. Journal of Computational Finance, 4(2), pp.63-68.
     """
 
     def __init__(self, call: bool = True, strike: float = 1.0):
@@ -78,7 +83,7 @@ class BSAmericanBinaryOption(BSModuleMixin):
         """
         return cls(call=derivative.call, strike=derivative.strike)
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         params = []
         if self.strike != 1.0:
             params.append(f"strike={self.strike}")
@@ -154,8 +159,8 @@ class BSAmericanBinaryOption(BSModuleMixin):
         d1 = self.d1(s, t, v)
         d2 = self.d2(s, t, v)
         c1 = self.N.cdf(d1 / sqrt2)
-        p1 = self.N.pdf(d1 / sqrt2)
-        p2 = self.N.pdf(d2 / sqrt2)
+        p1 = self.N.log_prob(d1 / sqrt2).exp()
+        p2 = self.N.log_prob(d2 / sqrt2).exp()
 
         d = (1 + c1 + (p1 + p2)) / (2 * self.strike)
 
@@ -232,7 +237,5 @@ class BSAmericanBinaryOption(BSModuleMixin):
 
 
 # Assign docstrings so they appear in Sphinx documentation
-BSAmericanBinaryOption.inputs = BSModuleMixin.inputs
-BSAmericanBinaryOption.inputs.__doc__ = BSModuleMixin.inputs.__doc__
-BSAmericanBinaryOption.forward = BSModuleMixin.forward
-BSAmericanBinaryOption.forward.__doc__ = BSModuleMixin.forward.__doc__
+set_attr_and_docstring(BSAmericanBinaryOption, "inputs", BSModuleMixin.inputs)
+set_attr_and_docstring(BSAmericanBinaryOption, "forward", BSModuleMixin.forward)
