@@ -2,6 +2,7 @@ import torch
 from torch import Tensor
 
 from pfhedge._utils.bisect import bisect
+from pfhedge._utils.doc import set_attr_and_docstring
 
 from ._base import BSModuleMixin
 
@@ -15,7 +16,7 @@ class BSEuropeanOption(BSModuleMixin):
 
     Shape:
         - Input : :math:`(N, *, 3)`, where :math:`*` means any number
-          of additional dimensions. See `inputs()` for the names of input features.
+          of additional dimensions. See ``inputs`` for the names of input features.
         - Output: :math:`(N, *, 1)`. Delta of the derivative.
           All but the last dimension are the same shape as the input.
 
@@ -26,7 +27,7 @@ class BSEuropeanOption(BSModuleMixin):
 
     Examples:
 
-        The `forward` method returns delta of the derivative.
+        The ``forward`` method returns delta of the derivative.
 
         >>> from pfhedge.nn import BSEuropeanOption
         >>>
@@ -41,6 +42,9 @@ class BSEuropeanOption(BSModuleMixin):
         tensor([[0.4497],
                 [0.5126],
                 [0.5752]])
+
+    References:
+        John C. Hull, 2003. Options futures and other derivatives. Pearson.
     """
 
     def __init__(self, call: bool = True, strike: float = 1.0):
@@ -71,7 +75,7 @@ class BSEuropeanOption(BSModuleMixin):
         """
         return cls(call=derivative.call, strike=derivative.strike)
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         params = []
         if not self.call:
             params.append(f"call={self.call}")
@@ -131,7 +135,7 @@ class BSEuropeanOption(BSModuleMixin):
 
         s, t, v = map(torch.as_tensor, (log_moneyness, expiry_time, volatility))
         price = self.strike * s.exp()
-        gamma = self.N.pdf(self.d1(s, t, v)) / (price * v * t.sqrt())
+        gamma = self.N.log_prob(self.d1(s, t, v)).exp() / (price * v * t.sqrt())
 
         return gamma
 
@@ -142,7 +146,6 @@ class BSEuropeanOption(BSModuleMixin):
 
         Args:
             log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            max_log_moneyness (torch.Tensor): Cumulative maximum of the log moneyness.
             expiry_time (torch.Tensor): Time to expiry of the option.
             volatility (torch.Tensor): Volatility of the underlying asset.
 
@@ -168,7 +171,11 @@ class BSEuropeanOption(BSModuleMixin):
         return price
 
     def implied_volatility(
-        self, log_moneyness: Tensor, expiry_time: Tensor, price: Tensor, precision=1e-6
+        self,
+        log_moneyness: Tensor,
+        expiry_time: Tensor,
+        price: Tensor,
+        precision: float = 1e-6,
     ) -> Tensor:
         """Returns implied volatility of the derivative.
 
@@ -194,7 +201,5 @@ class BSEuropeanOption(BSModuleMixin):
 
 
 # Assign docstrings so they appear in Sphinx documentation
-BSEuropeanOption.inputs = BSModuleMixin.inputs
-BSEuropeanOption.inputs.__doc__ = BSModuleMixin.inputs.__doc__
-BSEuropeanOption.forward = BSModuleMixin.forward
-BSEuropeanOption.forward.__doc__ = BSModuleMixin.forward.__doc__
+set_attr_and_docstring(BSEuropeanOption, "inputs", BSModuleMixin.inputs)
+set_attr_and_docstring(BSEuropeanOption, "forward", BSModuleMixin.forward)
