@@ -1,8 +1,12 @@
+from typing import List
+
 import torch
 from torch import Tensor
 
 import pfhedge.autogreek as autogreek
 from pfhedge._utils.bisect import bisect
+from pfhedge._utils.doc import set_attr_and_docstring
+from pfhedge._utils.doc import set_docstring
 
 from ._base import BSModuleMixin
 
@@ -16,7 +20,7 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
     Shape:
         - Input: :math:`(N, *, 3)`, where :math:`*` means any number of additional
-          dimensions. See `inputs()` for the names of input features.
+          dimensions. See :func:`inputs` for the names of input features.
         - Output: :math:`(N, *, 1)` Delta of the derivative.
           All but the last dimension are the same shape as the input.
 
@@ -27,7 +31,7 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
     Examples:
 
-        The `forward` method returns delta of the derivative.
+        The ``forward`` method returns delta of the derivative.
 
         >>> from pfhedge.nn import BSEuropeanBinaryOption
         >>>
@@ -42,6 +46,9 @@ class BSEuropeanBinaryOption(BSModuleMixin):
         tensor([[6.2576],
                 [6.3047],
                 [6.1953]])
+
+    References:
+        John C. Hull, 2003. Options futures and other derivatives. Pearson.
     """
 
     def __init__(self, call: bool = True, strike: float = 1.0):
@@ -77,13 +84,13 @@ class BSEuropeanBinaryOption(BSModuleMixin):
         """
         return cls(call=derivative.call, strike=derivative.strike)
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         params = []
         if self.strike != 1.0:
             params.append(f"strike={self.strike}")
         return ", ".join(params)
 
-    def inputs(self) -> list:
+    def inputs(self) -> List[str]:
         return ["log_moneyness", "expiry_time", "volatility"]
 
     def price(
@@ -134,7 +141,9 @@ class BSEuropeanBinaryOption(BSModuleMixin):
         """
         s, t, v = map(torch.as_tensor, (log_moneyness, expiry_time, volatility))
 
-        delta = self.N.pdf(self.d2(s, t, v)) / (self.strike * s.exp() * v * t.sqrt())
+        delta = self.N.log_prob(self.d2(s, t, v)).exp() / (
+            self.strike * s.exp() * v * t.sqrt()
+        )
         return delta
 
     def gamma(
@@ -193,6 +202,5 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
 
 # Assign docstrings so they appear in Sphinx documentation
-BSEuropeanBinaryOption.inputs.__doc__ = BSModuleMixin.inputs.__doc__
-BSEuropeanBinaryOption.forward = BSModuleMixin.forward
-BSEuropeanBinaryOption.forward.__doc__ = BSModuleMixin.forward.__doc__
+set_docstring(BSEuropeanBinaryOption, "inputs", BSModuleMixin.inputs)
+set_attr_and_docstring(BSEuropeanBinaryOption, "forward", BSModuleMixin.forward)
