@@ -1,3 +1,4 @@
+from typing import List
 from typing import Optional
 
 import torch
@@ -5,6 +6,8 @@ from torch import Tensor
 
 import pfhedge.autogreek as autogreek
 from pfhedge._utils.bisect import bisect
+from pfhedge._utils.doc import set_attr_and_docstring
+from pfhedge._utils.doc import set_docstring
 
 from ._base import BSModuleMixin
 
@@ -18,7 +21,7 @@ class BSLookbackOption(BSModuleMixin):
 
     Shape:
         - Input: :math:`(N, *, 4)`, where :math:`*` means any number of additional
-          dimensions.  See `inputs()` for the names of input features.
+          dimensions.  See :func:`inputs` for the names of input features.
         - Output: :math:`(N, *, 1)`. Delta of the derivative.
           All but the last dimension are the same shape as the input.
 
@@ -29,7 +32,7 @@ class BSLookbackOption(BSModuleMixin):
 
     Examples:
 
-        The `forward` method returns delta of the derivative.
+        The ``forward`` method returns delta of the derivative.
 
         >>> from pfhedge.nn import BSLookbackOption
         >>>
@@ -44,6 +47,10 @@ class BSLookbackOption(BSModuleMixin):
         tensor([[0.9208],
                 [1.0515],
                 [1.0515]])
+
+    References:
+        Conze, A., 1991. Path dependent options: The case of lookback options.
+        The Journal of Finance, 46(5), pp.1893-1907.
     """
 
     def __init__(self, call: bool = True, strike: float = 1.0):
@@ -79,13 +86,13 @@ class BSLookbackOption(BSModuleMixin):
         """
         return cls(call=derivative.call, strike=derivative.strike)
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         params = []
         if self.strike != 1.0:
             params.append(f"strike={self.strike}")
         return ", ".join(params)
 
-    def inputs(self) -> list:
+    def inputs(self) -> List[str]:
         return ["log_moneyness", "max_log_moneyness", "expiry_time", "volatility"]
 
     def price(
@@ -126,7 +133,7 @@ class BSLookbackOption(BSModuleMixin):
         price_0 = self.strike * (
             s.exp() * self.N.cdf(d1)
             - self.N.cdf(d2)
-            + s.exp() * v * t.sqrt() * (d1 * self.N.cdf(d1) + self.N.pdf(d1))
+            + s.exp() * v * t.sqrt() * (d1 * self.N.cdf(d1) + self.N.log_prob(d1).exp())
         )
         # when max moneyness >= strike
         price_1 = self.strike * (
@@ -134,7 +141,7 @@ class BSLookbackOption(BSModuleMixin):
             - m.exp() * self.N.cdf(e2)
             + m.exp()
             - 1
-            + s.exp() * v * t.sqrt() * (e1 * self.N.cdf(e1) + self.N.pdf(e1))
+            + s.exp() * v * t.sqrt() * (e1 * self.N.cdf(e1) + self.N.log_prob(e1).exp())
         )
 
         return torch.where(m < 0, price_0, price_1)
@@ -249,6 +256,5 @@ class BSLookbackOption(BSModuleMixin):
 
 
 # Assign docstrings so they appear in Sphinx documentation
-BSLookbackOption.inputs.__doc__ = BSModuleMixin.inputs.__doc__
-BSLookbackOption.forward = BSModuleMixin.forward
-BSLookbackOption.forward.__doc__ = BSModuleMixin.forward.__doc__
+set_docstring(BSLookbackOption, "inputs", BSModuleMixin.inputs)
+set_attr_and_docstring(BSLookbackOption, "forward", BSModuleMixin.forward)
