@@ -1,4 +1,5 @@
-import torch
+from typing import Optional
+
 from torch import Tensor
 from torch.nn import Module
 
@@ -7,7 +8,7 @@ from ..functional import leaky_clamp
 
 
 class LeakyClamp(Module):
-    """Leakily Clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+    """Leakily Clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
     The bounds :math:`\\min` and :math:`\\max` can be tensors.
 
@@ -23,7 +24,7 @@ class LeakyClamp(Module):
         \\max < \\text{input}
         \\end{cases}
 
-    Otherwise:
+    If :math:`\\min > \\max`:
 
     .. math ::
 
@@ -42,6 +43,7 @@ class LeakyClamp(Module):
 
     Examples:
 
+        >>> import torch
         >>> from pfhedge.nn import LeakyClamp
         >>> m = LeakyClamp()
         >>> input = torch.linspace(-2, 12, 15) * 0.1
@@ -60,11 +62,13 @@ class LeakyClamp(Module):
         super().__init__()
         self.clamped_slope = clamped_slope
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return f"clamped_slope={self.clamped_slope}" if self.clamped_slope != 0 else ""
 
-    def forward(self, input: Tensor, min: Tensor = None, max: Tensor = None) -> Tensor:
-        """Clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+    def forward(
+        self, input: Tensor, min: Optional[Tensor] = None, max: Optional[Tensor] = None
+    ) -> Tensor:
+        """Clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
         Args:
             input (torch.Tensor): The input tensor.
@@ -85,7 +89,7 @@ class LeakyClamp(Module):
 
 
 class Clamp(Module):
-    """Clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+    """Clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
     The bounds :math:`\\min` and :math:`\\max` can be tensors.
 
@@ -99,7 +103,7 @@ class Clamp(Module):
         \\max & \\max < \\text{input}
         \\end{cases}
 
-    Otherwise:
+    If :math:`\\min > \\max`:
 
     .. math ::
 
@@ -114,7 +118,9 @@ class Clamp(Module):
 
     Examples:
 
+        >>> import torch
         >>> from pfhedge.nn import Clamp
+        >>>
         >>> m = Clamp()
         >>> input = torch.linspace(-2, 12, 15) * 0.1
         >>> input
@@ -135,8 +141,10 @@ class Clamp(Module):
         tensor([0.0000, 0.5000])
     """
 
-    def forward(self, input: Tensor, min: Tensor = None, max: Tensor = None) -> Tensor:
-        """Clamp all elements in `input` into the range :math:`[\\min, \\max]`.
+    def forward(
+        self, input: Tensor, min: Optional[Tensor] = None, max: Optional[Tensor] = None
+    ) -> Tensor:
+        """Clamp all elements in ``input`` into the range :math:`[\\min, \\max]`.
 
         Args:
             input (torch.Tensor): The input tensor.
