@@ -1,4 +1,5 @@
 from abc import ABC
+from typing import Callable
 
 import torch
 from torch import Tensor
@@ -36,7 +37,7 @@ class HedgeLoss(Module, ABC):
         """Returns the cash amount which is as preferable as
         the given profit-loss distribution in terms of the loss.
 
-        The output `cash` is expected to satisfy the following relation:
+        The output ``cash`` is expected to satisfy the following relation:
 
         .. code::
 
@@ -63,7 +64,8 @@ class HedgeLoss(Module, ABC):
 class EntropicRiskMeasure(HedgeLoss):
     """Creates a loss given by the entropic risk measure.
 
-    The entropic risk measure of the profit-loss distribution `pnl` is given by:
+    The entropic risk measure of the profit-loss distribution
+    :math:`\\text{pnl}` is given by:
 
     .. math ::
 
@@ -100,7 +102,7 @@ class EntropicRiskMeasure(HedgeLoss):
         super().__init__()
         self.a = a
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return f"a={self.a}" if self.a != 1 else ""
 
     def forward(self, input: Tensor) -> Tensor:
@@ -126,7 +128,7 @@ class EntropicRiskMeasure(HedgeLoss):
 class EntropicLoss(HedgeLoss):
     """Creates a loss given by the negative of expected exponential utility.
 
-    The loss of the profit-loss `pnl` is given by
+    The loss of the profit-loss :math:`\\text{pnl}` is given by:
 
     .. math ::
 
@@ -162,7 +164,7 @@ class EntropicLoss(HedgeLoss):
         super().__init__()
         self.a = a
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return f"a={self.a}" if self.a != 1 else ""
 
     def forward(self, input: Tensor) -> Tensor:
@@ -189,7 +191,7 @@ class EntropicLoss(HedgeLoss):
 class IsoelasticLoss(HedgeLoss):
     """Creates a loss function that measures the isoelastic utility.
 
-    The loss of the profit-loss distribution `pnl` is given by
+    The loss of the profit-loss :math:`\\text{pnl}` is given by:
 
     .. math ::
 
@@ -237,7 +239,7 @@ class IsoelasticLoss(HedgeLoss):
         super().__init__()
         self.a = a
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return f"a={self.a}"
 
     def forward(self, input: Tensor) -> Tensor:
@@ -275,7 +277,7 @@ class ExpectedShortfall(HedgeLoss):
         super().__init__()
         self.p = p
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return str(self.p)
 
     def forward(self, input: Tensor) -> Tensor:
@@ -319,13 +321,13 @@ class OCE(HedgeLoss):
         tensor(-0.0821)
     """
 
-    def __init__(self, utility):
+    def __init__(self, utility: Callable[[Tensor], Tensor]) -> None:
         super().__init__()
 
         self.utility = utility
         self.w = Parameter(torch.tensor(0.0))
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return self.utility.__name__ + f", w={self.w}"
 
     def forward(self, input: Tensor) -> Tensor:
