@@ -1,4 +1,5 @@
-import torch
+from typing import List
+
 from torch import Tensor
 from torch.nn import Module
 
@@ -9,7 +10,7 @@ from .clamp import Clamp
 class WhalleyWilmott(Module):
     """Initialize Whalley-Wilmott's hedging strategy of a derivative.
 
-    The `forward` method returns the next hedge ratio.
+    The ``forward`` method returns the next hedge ratio.
 
     This is the optimal hedging strategy for asymptotically small transaction cost.
 
@@ -19,19 +20,22 @@ class WhalleyWilmott(Module):
 
     Shape:
         - Input: :math:`(N, *, H_{\\text{in}})`.  Here, :math:`*` means any number of
-          additional dimensions and `H_in` is the number of input features.
-          See `inputs()` for the names of input features.
+          additional dimensions and :math:`H_{\\text{in}}` is
+          the number of input features.
+          See :func:`inputs()` for the names of input features.
         - Output: :math:`(N, *, 1)`. The hedge ratio at the next time step.
 
     Examples:
 
         An example for :class:`pfhedge.instruments.EuropeanOption`.
 
+        >>> import torch
         >>> from pfhedge.nn import WhalleyWilmott
         >>> from pfhedge.instruments import BrownianStock
         >>> from pfhedge.instruments import EuropeanOption
-        >>> deriv = EuropeanOption(BrownianStock(cost=1e-5))
-        >>> m = WhalleyWilmott(deriv)
+        >>> derivative = EuropeanOption(BrownianStock(cost=1e-5))
+        >>>
+        >>> m = WhalleyWilmott(derivative)
         >>> m.inputs()
         ['log_moneyness', 'expiry_time', 'volatility', 'prev_hedge']
         >>> input = torch.tensor([
@@ -49,8 +53,8 @@ class WhalleyWilmott(Module):
 
         An example for :class:`pfhedge.instruments.EuropeanOption` without cost.
 
-        >>> deriv = EuropeanOption(BrownianStock())
-        >>> m = WhalleyWilmott(deriv)
+        >>> derivative = EuropeanOption(BrownianStock())
+        >>> m = WhalleyWilmott(derivative)
         >>> m.inputs()
         ['log_moneyness', 'expiry_time', 'volatility', 'prev_hedge']
         >>> input = torch.tensor([
@@ -67,11 +71,12 @@ class WhalleyWilmott(Module):
                 [0.7945]])
 
     References:
-        - Whalley, A.E. and Wilmott, P., An asymptotic analysis of an optimal hedging model
-          for option pricing with transaction costs. Mathematical Finance, 1997, 7, 307–324.
+        - Whalley, A.E. and Wilmott, P., An asymptotic analysis of an optimal hedging
+          model for option pricing with transaction costs. Mathematical Finance,
+          1997, 7, 307–324.
     """
 
-    def __init__(self, derivative, a: float = 1.0):
+    def __init__(self, derivative, a: float = 1.0) -> None:
         super().__init__()
         self.derivative = derivative
         self.a = a
@@ -79,7 +84,7 @@ class WhalleyWilmott(Module):
         self.bs = BlackScholes(derivative)
         self.clamp = Clamp()
 
-    def inputs(self) -> list:
+    def inputs(self) -> List[str]:
         """Returns the names of input features.
 
         Returns:
@@ -87,7 +92,7 @@ class WhalleyWilmott(Module):
         """
         return self.bs.inputs() + ["prev_hedge"]
 
-    def extra_repr(self):
+    def extra_repr(self) -> str:
         return f"a={self.a}" if self.a != 1 else ""
 
     def forward(self, input: Tensor) -> Tensor:
@@ -104,7 +109,7 @@ class WhalleyWilmott(Module):
         """Returns half-width of the no-transaction band.
 
         Args:
-            input : Tensor
+            input (Tensor): The input tensor.
 
         Shape:
             - Input: :math:`(N, *, H_{\\text{in}} - 1)`
@@ -115,8 +120,8 @@ class WhalleyWilmott(Module):
         """
         cost = self.derivative.underlier.cost
 
-        price = self.derivative.strike * torch.exp(input[..., [0]])
-        gamma = self.bs.gamma(*(input[..., [i]] for i in range(input.size()[-1])))
-        width = (cost * (3 / 2) * (gamma ** 2) * price / self.a) ** (1 / 3)
+        spot = self.derivative.strike * input[..., [0]].exp()
+        gamma = self.bs.gamma(*(input[..., [i]] for i in range(input.size(-1))))
+        width = (cost * (3 / 2) * (gamma ** 2) * spot / self.a) ** (1 / 3)
 
         return width
